@@ -1,14 +1,19 @@
-# Copyrights 2011-2014 by [Mark Overmeer].
+# Copyrights 2011-2018 by [Mark Overmeer].
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
-# Pod stripped from pm file by OODoc 2.01.
-use warnings;
-use strict;
+# Pod stripped from pm file by OODoc 2.02.
+# This code is part of distribution Any-Daemon. Meta-POD processed with
+# OODoc into POD and HTML manual-pages.  See README.md
+# Copyright Mark Overmeer.  Licensed under the same terms as Perl itself.
 
 package Any::Daemon;
-our $VERSION = '0.94';
+use vars '$VERSION';
+$VERSION = '0.95';
 
 
+use warnings;
+use strict;
+
 use Log::Report::Optional  'any-daemon';
 
 use POSIX         qw(setsid setuid setgid :sys_wait_h);
@@ -29,7 +34,6 @@ sub new(@) {my $class = shift; (bless {}, $class)->init({@_})}
 
 sub init($)
 {   my ($self, $args) = @_;
-
     $self->{AD_pidfn} = $args->{pid_file};
 
     my $user = $args->{user};
@@ -41,6 +45,9 @@ sub init($)
         }
         else { $self->{AD_uid} = $user }
     }
+    elsif($EUID==0)
+    {   warning __"running daemon as root is dangerous: please specify user";
+    }
 
     my $group = $args->{group};
     if(defined $group)
@@ -75,7 +82,9 @@ sub run(@)
 
     my $bg = exists $args{background} ? $args{background} : 1;
     if($bg)
-    {   my $kid = fork;
+    {   trace "backgrounding managing daemon";
+
+        my $kid = fork;
         if($kid)
         {   # starting parent is ready to leave
             exit 0;
@@ -113,9 +122,6 @@ sub run(@)
         $@ and error __x"cannot switch to user/group to {uid}/{gid}: {err}"
           , uid => $uid, gid => $gid, err => $@;
     }
-    elsif($EUID==0)
-    {   warning __"running daemon as root is dangerous: please specify user";
-    }
 
     my $sid         = setsid;
 
@@ -127,7 +133,7 @@ sub run(@)
 
     my $run_child   = sub
       { # re-seed the random number sequence per process
-        srand;
+        srand(time+$$);
 
         # unhandled errors are to be treated seriously.
         my $rc = try { $child_task->(@_) };
@@ -166,7 +172,7 @@ sub run(@)
         open STDERR, '>', File::Spec->devnull;
     }
 
-    info __x"daemon started; proc={proc} uid={uid} gid={gid} childs={max}"
+    notice __x"daemon started; proc={proc} uid={uid} gid={gid} childs={max}"
       , proc => $PID, uid => $EUID, gid => $EGID, max => $max_childs;
 
     $child_died->($max_childs, $run_child);
@@ -175,7 +181,7 @@ sub run(@)
     sleep 60 while 1;
 }
 
-sub _reconfing_daemon(@)
+sub _reconfig_daemon(@)
 {   my @childs = @_;
     notice "HUP: reconfigure deamon not implemented";
 }
@@ -203,7 +209,8 @@ sub _child_died($$)
         last ZOMBIE if $kid <= 0;
 
         if($? != 0)
-        {   notice "$kid process died with errno $?";
+        {   my $err = WIFEXITED($?) ? "errno ".WEXITSTATUS($?) : "sig $?";
+            notice "$kid process terminated with $err";
             # when children start to die, do not respawn too fast,
             # because usually this means serious troubles with the
             # server (like database) or implementation.
